@@ -9,8 +9,8 @@ use std::char::from_u32;
 /// Set to `true` for λ or `false` for \ when displaying lambda terms. The default is `true`.
 pub const DISPLAY_PRETTY: bool = true;
 
-/// Set to `true` for classic lambda mode or `false` for the De Bruijn index display mode when
-/// displaying lambda terms. The default is `false`.
+/// Set to `true` for classic lambda calculus notation mode or `false` for the De Bruijn index
+/// mode when displaying lambda terms. The default is `false`.
 pub const DISPLAY_CLASSIC: bool = false;
 
 /// A lambda term that is either a variable with a De Bruijn index, an abstraction over a term or
@@ -255,7 +255,7 @@ impl Term {
 /// ```
 pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 
-/// Produces an `App`lication of its `Term` arguments without any reduction, consuming them in the
+/// Produces an `App`lication of 2 given `Term`s without any reduction, consuming them in the
 /// process.
 ///
 /// # Example
@@ -288,7 +288,10 @@ pub fn show_precedence(term: &Term, context_precedence: usize, depth: u32) -> St
         },
         Abs(ref t) => {
             let ret = if DISPLAY_CLASSIC {
-                format!("{}{}. {}", if DISPLAY_PRETTY { 'λ' } else { '\\' }, from_u32(depth + 97).unwrap(), show_precedence(t, 0, depth + 1))
+                format!("{}{}. {}", if DISPLAY_PRETTY { 'λ' } else { '\\' },
+                    from_u32(depth + 97).unwrap(),
+                    show_precedence(t, 0, depth + 1)
+                )
             } else {
                 format!("{}{}", if DISPLAY_PRETTY { 'λ' } else { '\\' }, t)
             };
@@ -343,7 +346,9 @@ mod test {
 
     #[test]
     fn app_macro() {
-        assert_eq!(&format!("{}", app!(succ(), app!(Var(1), Var(2), Var(3)))), "(λλλ2(321))(123)");
+        assert_eq!(app!(succ(), app!(Var(1), Var(2), Var(3))),
+                   succ().app(Var(1).app(Var(2)).app(Var(3)))
+        );
     }
 
     #[test]
